object Glyph
with Lists +Outlines +Glyph_Images +Real_Numbers +Real_Vectors
body-with +Render_Glyphs
  
attributes
Name     : String; Create Get	-- Unique name, used for file name.
Bounds   : Box; Get		-- Dimensions of the glyph.
Outlines : Outline_List; Get	-- Defining the glyph's appearance.
Image    : Glyph_Image;	-- Cached raster image of the glyph.

procedure Add_Outline
Add : in Outline; -- Add this outline to the glyph.
-- Add an outline to a glyph.
Outline_Lists.Push(This.Outlines, Add);

function Get_Image
Staff_Height : in Real;  -- Height of staff in pixels.
Aspect_Ratio : in Real;  -- Zoom factor for X resolution.
Anti_Alias : in Positive; -- Factor for vertical anti-alias.
return Glyph_Image; -- The image of the glyph.
-- Get a raster image of this glyph. Create one if not cached.
if This.Image = null
  or else This.Image.Zoom /= Staff_Height
  or else This.Image.Aspect_Ratio /= Aspect_Ratio
  or else This.Image.Anti_Alias /= Anti_Alias
then 
   This.Image := Render(This, Staff_Height, Aspect_Ratio, Anti_Alias);
end if;
return This.Image;

procedure Set_Bounds
Left, Bottom : in Real; -- Coordinate minima.
Right, Top : in Real; -- Coordinate maxima.
-- Update the bounding box of the glyph.
This.Bounds := Box'(Left, Bottom, Right, Top);

function Get_Width 
return Real -- The width of the glyph.
-- Calculate the width of this glyph's bounding box.
return Get_Width(This.Bounds);

function Get_Height 
return Real -- The height of the glyph.
-- Calculate the height of this glyph's bounding box.
return Get_Height(This.Bounds);

pre-public
-- Internal package for lists of outlines.
package Outline_Lists is new Lists(Outlines.Outline);

-- A list of outlines to store a glyph's appearance.
subtype Outline_List is Outline_Lists.List;
